# Copyright (C) 2003-2005
# Emmanuel Saracco <esaracco@users.labs.libre-entreprise.org>
# Easter-eggs <http://www.easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307, USA.

do '../web-lib.pl';
&init_config();

# retreive ACLs for this module
my %ACLs = &get_module_acl ();

# global array for bad perl modules
# dependencies
my @perl_deps = ();

eval "use POSIX";
push @perl_deps, 'POSIX' if ($@ ne '');
eval "use File::Basename";
push @perl_deps, 'File::Basename' if ($@ ne '');
eval "use File::Find";
push @perl_deps, 'File::File' if ($@ ne '');
eval "use Date::Manip";
push @perl_deps, 'Date::Manip' if ($@ ne '');
eval "use Compress::Zlib;";
push @perl_deps, 'Compress::Zlib;' if ($@ ne '');
eval "use HTML::Entities;";
push @perl_deps, 'HTML::Entities;' if ($@ ne '');

# min stable clamav version supported by this module
my $supported_versions = '0.84';

# min devel (CVS) clamav version supported by this module
my $min_devel_date = '20050429';

# max items to display/page for quarantine search
# result
my $PAGE_ITEMS = 50;

# freshclam configuration
my %freshclam_config = ();

# commands for remote control of
# clamav daemon
%clamav_remote_actions = (
  'PING' => 0,
  'VERSION' => 0,
  'RELOAD' => 0,
  'SHUTDOWN' => 0,
  'SCAN' => 1,
  'RAWSCAN' => 1,
  'CONTSCAN' => 1,
# 'STREAM' => 0,
# 'SESSION' => 0,
# 'END' => 0
);

# freshclam predefined variables
# name => [1 = need value | 0 = alone]
my %freshclam_predefined = (
  'Example' => 0,
  'DatabaseOwner' => 1,
  'AllowSupplementaryGroups' => 0,
  'DatabaseDirectory' => 1,
  'Checks' => 1,
  'UpdateLogFile' => 1,
  'LogSyslog' => 0,
  'LogFacility' => 1,
  'LogVerbose' => 0,
  'DNSDatabaseInfo' => 1,
  'DatabaseMirror' => 1,
  'MaxAttempts' => 1,
  'HTTPProxyServer' => 1,
  'HTTPProxyPort' => 1,
  'HTTPProxyUsername' => 1,
  'HTTPProxyPassword' => 1,
  'LocalIPAddress' => 1,
  'NotifyClamd' => 0,
  'OnUpdateExecute' => 1,
  'OnErrorExecute' => 1
);

# clamav configuration
my %clamav_config = ();

# clamav predefined variables
# name => [1 = need value | 0 = alone]
my %clamav_predefined = (
  'LogFile' => 1,
  'LogFileUnlock' => 0,
  'LogFileMaxSize' => 1,
  'LogTime' => 0,
  'LogClean' => 0,
  'LogSyslog' => 0,
  'LogFacility' => 1,
  'LogVerbose' => 0,
  'PidFile' => 1,
  'DataDirectory' => 1,
  'LocalSocket' => 1,
  'FixStaleSocket' => 0,
  'TCPSocket' => 1,
  'TCPAddr' => 1,
  'MaxConnectionQueueLength' => 1,
  'MaxThreads' => 1,
  'ReadTimeout' => 1,
  'MaxDirectoryRecursion' => 1,
  'FollowDirectorySymlinks' => 0,
  'FollowFileSymlinks' => 0,
  'SelfCheck' => 1,
  'VirusEvent' => 1,
  'ExitOnOOM' => 0,
  'User' => 1,
  'AllowSupplementaryGroups' => 0,
  'Foreground' => 0,
  'Debug' => 0,
  'StreamMaxLength' => 1,
  'StreamMinPort' => 1,
  'StreamMaxPort' => 1,
  'DisableDefaultScanOptions' => 0,
#  'LeaveTemporaryFiles' => 0,
#  'StreamSaveToDisk' => 0,
  'ScanPE' => 0,
  'DetectBrokenExecutables' => 0,
  'ScanOLE2' => 0,
  'ScanHTML' => 0,
  'ScanMail' => 0,
  'MailFollowURLs' => 0,
  'ScanArchive' => 0,
  'ScanRAR' => 0,
  'ArchiveMaxFileSize' => 1,
  'ArchiveMaxRecursion' => 1,
  'ArchiveMaxFiles' => 1,
  'ArchiveMaxCompressionRatio' => 1,
  'ArchiveLimitMemoryUsage' => 0,
  'ArchiveBlockEncrypted' => 0,
  'ArchiveBlockMax' => 0,
#  'ArchiveDetectEncrypted' => 0,
  'ClamukoScanOnLine' => 0,
  'ClamukoScanOnOpen' => 0,
  'ClamukoScanOnClose' => 0,
  'ClamukoScanOnExec' => 0,
  'ClamukoIncludePath' => 1,
  'ClamukoExcludePath' => 0,
  'ClamukoMaxFileSize' => 1,
  'ClamukoScanArchive' => 0
);

# week days
my @days = (
  $text{'SUNDAY'},
  $text{'MONDAY'},
  $text{'TUESDAY'},
  $text{'WEDNESDAY'},
  $text{'THURSDAY'},
  $text{'FRIDAY'},
  $text{'SATURDAY'}
);

# all binaries pathes are here
my $all_path = ();
&clamav_init_all_path ();

# clean config inputs
&clamav_trim_config ();

sub clamav_get_acl ( $ )
{
  my $acl = shift;

  return $ACLs{$acl};
}

sub clamav_check_acl ( $ )
{
  my $acl = shift;

  if (($ACLs{$acl} != 1))
  {
    &header($text{'FORM_TITLE'}, "", undef, 1, 0);
    print qq(<hr>);
    &clamav_check_config_exit ($text{'MSG_ACL_DENIED'})
  }
}

# clamav_is_mailscanner ()
# IN: -
# OUT: TRUE if mailscanner is installed
#
# Check if mailscanner is installed
# 
sub clamav_is_mailscanner ()
{
  return ($config{'clamav_quarantine_soft'} == 4 &&
    $all_path{'MailScanner'} ne '');
}

# clamav_is_milter ()
# IN: -
# OUT: TRUE if clamav-milter is installed
#
# Check if clamav-milter is installed
# 
sub clamav_is_milter ()
{
  return ($config{'clamav_quarantine_soft'} == 3 && 
    $all_path{'clamav-milter'} ne '');
}

# clamav_is_amavis_ng ()
# IN: -
# OUT: TRUE if amavis-ng is installed
#
# Check if amavis-ng is installed
# 
sub clamav_is_amavis_ng ()
{
  return ($config{'clamav_quarantine_soft'} == 2 &&
    $all_path{'amavis-ng'} ne '');
}

# clamav_is_amavisd_new ()
# IN: -
# OUT: TRUE if amavisd-new is installed
#
# Check if amavisd-new is installed
#
sub clamav_is_amavisd_new ()
{
  return ($config{'clamav_quarantine_soft'} == 1 && 
    (
      $all_path{'amavisd-new'} ne '' || 
      $all_path{'amavisd'} ne '' ||
      $all_path{'amavis'} ne ''
    )
  );
}

# clamav_check_config ()
# IN: -
# OUT: -
#
# check if the module config is ok. if not, display a warning
# and exit
# 
sub clamav_main_check_config
{
  my $ok = 0;
  my $msg = '';
    
  # touch log files
  if ($config{'clamav_clamav_log'} ne '')
  {
    system ("$all_path{'touch'}touch $config{'clamav_clamav_log'}") ;
    system ("$all_path{'chmod'}chmod clamav.clamav $config{'clamav_clamav_log'}");
  }
  if ($config{'clamav_freshclam_log'} ne '')
  {
    system ("$all_path{'touch'}touch $config{'clamav_freshclam_log'}");
    system ("$all_path{'chmod'}chmod clamav.clamav $config{'clamav_freshclam_log'}");
  }
  
  # do tests
  if ($all_path{'clamscan'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMSCAN'};
  }
  elsif ($all_path{'freshclam'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_REFRESH'};
  }
  elsif (!&clamav_check_version ())
  {
     $msg = $text{'MSG_BAD_CLAMAV_VERSION'};
  }
  elsif ($config{'clamav_working_path'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_WORKING_PATH'};
  }
  elsif ($config{'clamav_init_script'} eq '' || 
    ! -x $config{'clamav_init_script'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_DAEMON'};
  }
  elsif ($config{'clamav_clamav_log'} eq '' ||
   ! -f $config{'clamav_clamav_log'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_LOG'};
  }
  elsif ($config{'clamav_clamav_conf'} eq '' ||
    ! -f $config{'clamav_clamav_conf'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_CONF'};
  }
  elsif ($config{'clamav_db1'} eq '' ||
    ! -f $config{'clamav_db1'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB1'};
  }
  elsif ($config{'clamav_db2'} eq '' ||
    ! -f $config{'clamav_db2'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB2'};
  }
  elsif ($config{'clamav_freshclam_log'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_LOG'};
  }
  elsif ($config{'clamav_refresh_use_cron'} == 0)
  {
    if ($config{'clamav_freshclam_conf'} eq '' ||
      ! -f $config{'clamav_freshclam_conf'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_CONF'};
    }
    elsif ($config{'clamav_freshclam_init_script'} eq '' ||
      ! -x $config{'clamav_freshclam_init_script'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_INIT'};
    }
    else
    {
      $ok = 1;
    }
  }
  elsif ($config{'clamav_refresh_use_cron'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_USE_CRON'};
  }
  else
  {
    $ok = 1;
  }

  # if there was a problem, exit displaying
  # a message
  &clamav_check_config_exit ($msg) if (!$ok);

  # create module's temporary directory
  system ("$all_path{'mkdir'}mkdir -p $config{'clamav_working_path'}/.clamav/$remote_user");
  # modify permissions
  system ("$all_path{'chmod'}chmod 700 /$config{'clamav_working_path'}/.clamav");
}

# clamav_init_all_path ()
# IN: -
# OUT: -
#
# Initialize binaries pathes
# 
sub clamav_init_all_path
{
  my @exe = (
    'amavis-ng',
    'amavisd-new',
    'amavisd',
    'amavis',
    'amavis-inject',
    'chmod',
    'clamav-milter',
    'clamscan',
    'cp',
    'du',
    'freshclam',
    'grep',
    'gzip',
    'gunzip',
    'hexdump',
    'ln',
    'MailScanner',
    'mkdir',
    'mv',
    'runlevel',
    'rm',
    'rsmtp',
    'sendmail',
    'sigtool',
    'strings',
    'touch'
  );

  foreach my $item (@exe)
  {
    $all_path{$item} = &clamav_has_command ($item);
  }
}

# clamav_get_proxy_settings ()
# IN: -
# OUT: A array with 0="proxy server" and 1="proxy port"
#
# Load the proxy settings and return them
# 
sub clamav_get_proxy_settings
{
  &clamav_load_freshclam_config ();

  return (
    "$freshclam_config{'HTTPProxyServer'}",
    "$freshclam_config{'HTTPProxyPort'}"
  )
    if (
      $freshclam_config{'HTTPProxyServer'} && 
      $freshclam_config{'HTTPProxyPort'}
    );

  return ();
}

# clamav_display_combo_freshclam_predefined ( $ )
# IN: 1 if combo must not display values that already exist
#     in the config file
# OUT: 1 if combo box has been displayed (if there is result)
#
# Display a combo box with freshclam predifined variables
# 
sub clamav_display_combo_freshclam_predefined ( $ )
{
  my $uniq = shift;
  my $buf = '';
  my $ok = 0;
  
  $buf = qq(<select name="nsfreshclam_add_key">\n);
  foreach $key (sort keys %freshclam_predefined)
  {
    if (!defined $freshclam_config{$key})
    {
      $ok = 1;
      $buf .= qq(<option value="$key">$key</option>\n)
    }
  }
  $buf .= qq(</select>\n);

  # only print if there is result
  print $buf if ($ok == 1);

  return $ok;
}

# clamav_display_combo_clamav_predefined ( $ )
# IN: 1 if combo must not display values that already exist
#     in the config file
# OUT: 1 if combo box has been displayed (if there is result)
#
# Display a combo box with clamav predifined variables
#
sub clamav_display_combo_clamav_predefined ($)
{
  my $uniq = shift;
  my $buf = '';
  my $ok = 0;
  
  $buf = qq(<select name="nsclamav_add_key">\n);
  foreach $key (sort keys %clamav_predefined)
  {
    if (!defined $clamav_config{$key})
    {
      $ok = 1;
      $buf .= qq(<option value="$key">$key</option>\n)
    }
  }
  $buf .= qq(</select>\n);

  # only print if there is result
  print $buf if ($ok == 1);

  return $ok;
}

# clamav_quarantine_resend_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for resending quarantined emails. 
# if not, display a warning and exit
# 
sub clamav_quarantine_resend_check_config
{
  my $ok = 0;
  my $msg = '';

  if (($all_path{'rsmtp'} eq '') && ($all_path{'sendmail'} eq ''))
  {
    $msg = $text{'MSG_CONFIG_ALERT_MTA'};
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (!$ok);
}

# clamav_has_command ( $ )
# IN: binary to test
# OUT: system path for this binary
#
# Search and return path for a given binary file
# -> clamav_init_all_path () need to be call before
#    this function
# 
sub clamav_has_command
{
  my $exe = shift;
  my @path = (
    '/bin/',
    '/sbin/',
    '/usr/bin/',
    '/usr/sbin/',
    '/usr/local/bin/',
    '/usr/local/sbin/',
    '/opt/bin/',
    '/opt/sbin/',
    '/opt/local/bin/',
    '/opt/local/sbin/'
  );

  if ($config{'clamav_bin'})
  {
    $config{'clamav_bin'} .= '/' if ($config{'clamav_bin'} !~ /\/$/);
    push (@path, $config{'clamav_bin'});
  }

  foreach my $item (@path)
  {
    return $item if (-e $item . $exe);  
  }
  
  return '';
}

# clamav_quarantine_main_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for managing quarantined emails. 
# if not, display a warning and exit
#
sub clamav_quarantine_main_check_config
{
  my $ok = 0;

  if ($config{'clamav_quarantine_soft'} == 0 ||
    (!&clamav_is_amavisd_new () &&
    !&clamav_is_amavis_ng () &&
    !&clamav_is_milter () &&
    !&clamav_is_mailscanner ()))
  {
    print qq(<p>$text{'MSG_CONFIG_ALERT_AMAVIS'}</p>);
  }
  elsif ($config{'clamav_quarantine'} eq '' ||
    ! -d $config{'clamav_quarantine'})
  {
    print qq(<p>$text{'MSG_CONFIG_ALERT_QUARANTINE'}</p>);
  }
  elsif (length ($config{'clamav_quarantine'}) < 8)
  {
    printf qq(<p>$text{'MSG_CONFIG_WARNING_QUARANTINE_TO_SHORT'}</p>),
      $config{'clamav_quarantine'};
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (!$ok);
}

# clamav_signatures_check_config ()
# IN: -
# OUT: -
#
# check if the server config is ok for the signatures creation section. 
# if not, display a warning and exit
#
sub clamav_signatures_check_config
{
  &clamav_check_config_exit ($text{'MSG_ERROR_SIGNATURES_CONFIG'})
    if (
      ($all_path{'strings'} eq '') ||
      ($all_path{'hexdump'} eq '')
    );
}

# clamav_check_config_exit ( $ )
# IN: Message to display
# OUT: -
#
# print a message and exit
# 
sub clamav_check_config_exit ( $ )
{
  my $msg = shift;

  print qq(<p>$msg</p>) if ($msg);
  print qq(<hr>);
  &footer("/", $text{'index'});

  exit (1);
}

# clamav_trim_config ()
# IN: -
# OUT: -
#
# remove trailing and pending spaces from every module's
# config variable
# 
sub clamav_trim_config
{
  foreach $key (keys %config)
  {
    $config{$key} =~ s/(^\s+|\s+$)//g;
  }
}

# clamav_html_cut ( $ $ )
# IN: the string to cut, the max length before cutting
# OUT: the cutted string with HTML <br> tags
#
# cut a string with HTML <br> tags
# 
sub clamav_html_cut ( $ $ )
{
  my ($string, $len) = @_;
  my $i = 0;
  my $new = '';

  for ($i = 0; $i < length ($string); $i++)
  {
    $new .= '<br>' if ($i > 0 && !($i % $len));
    $new .= substr ($string, $i, 1);
  }

  return $new;
}

# clamav_check_signature ( $ )
# IN: new virus signature
# OUT: 1 => the signature begun with 00
#      2 => the signature is not between 40-200 characters
#      3 => signature is ok
#
# check the ClamAV compatibility for a given virus signature
#      
sub clamav_check_signature ( $ )
{
  my $signature = shift;
  my $ret = 3;

  if ($signature =~ /^00/)
  {
    $ret = 1;
  }
  elsif (length ($signature) < 40 || length ($signature) > 200)
  {
    $ret = 2;
  }

  return $ret;
}

# clamav_get_version ()
# IN: -
# OUT: clamav release
#
# return the current installed clamav release
# 
sub clamav_get_version
{
  my $ret = '';

  $out = `$all_path{'clamscan'}clamscan --version 2>&1`;

  # clamav CVS or clamav < 0.80 or clamav >= 0.80
  if ($out =~ /devel-([\d+]{8})/ || $out =~ /(\d+\.\d+)/ || 
    $out =~ /ClamAV\s+(\d+\.\d+)/) {$ret = $1}

print $ret;

  return $ret;
}

# clamav_check_version ()
# IN: -
# OUT: boolean value
#
# return true if the installed version of clamav
# is supported by this module. if this is a "stable" release, we look
# at its version number, otherwise if it is a CVS release we look at its
# date
# 
sub clamav_check_version
{
  my $out = '';
  my $ret = 0;

  $out = `$all_path{'clamscan'}clamscan --version 2>&1`;

  # clamav CVS
  if ($out =~ /devel-([\d+]{8})/)
    {$ret = (&Date_Cmp ($min_devel_date, $1) <= 0)}
  # clamav < 0.84 or clamav >= 0.84
  elsif ($out =~ /(\d+\.\d+)/ || $out =~ /ClamAV\s+(\d+\.\d+)/)
    {$ret = ($1 >= $supported_versions)}

  return $ret;
}

# clamav_get_runlevel ()
# IN: -
# OUT: the current runlevel
#
# return the current runlevel of the server system
# 
sub clamav_get_runlevel
{
  my $runlevel = '';

  $runlevel = `$all_path{'runlevel'}runlevel`;
  $runlevel =~ s/N //;
  $runlevel =~ s/(^\s+|\s+$)//g;

  $runlevel = int ($runlevel);
  $runlevel = '' if ($runlevel == 0);

  return $runlevel;
}

# clamav_get_freshclam_daemon_settings ()
# IN: -
# OUT: check period
#
# Return the check period specified in the configuration file of
# freshclam daemon
#
sub clamav_get_freshclam_daemon_settings
{
  my $conf = '';
  my $ret = '';
  
  &clamav_load_freshclam_config ();
  
  $ret = $freshclam_config{'Checks'};
  $ret = 1 if $ret eq '';

  return $ret;
}

# clamav_vdb_get_pure_name ( $ )
# IN: ClamAV registered virus name
# OUT: Simple virus name
#
# Try to guess the generic name of the given virus
# 
sub clamav_vdb_get_pure_name ( $ )
{
  my $name = shift;
  my ($b, $m, $e);

  if ($name =~ /^(.*)[\.\-](.*)[\.\-](.*)$/)
  {
    ($b, $m, $e) = ($1, $2, $3);
    
      return ($m !~ /[a-z,A-Z]/ || length ($m) == 1) ?
        &clamav_vdb_get_pure_name ("$b") : $m;
  }

  if ($name =~ /^(.*)[\.\-](.*)$/)
  {
    ($b, $e) = ($1, $2);

    if ($e !~ /[a-z,A-Z]/) {return $b;} elsif ($e) {return $e;}
  }

  if ($name =~ /^(.*)[\.\-].*$/)
  {
    return $1;
  }

  return $name;
}

# clamav_vdb_search ( $ $ )
# IN: virus name, TRUE if we must do a strict search
# OUT: -
#
# Do a search in ClamAV database and display the result
# 
sub clamav_vdb_search ( $ $ )
{
  my ($virus, $strict, $case, $sortr) = @_;
  my $string = '';
  my $escaped = '';
  my $first = 1;

  exit (1) if ($virus =~ /;/);

  # case sensitive search?
  $case = ($case) ? ' ' : ' -i ';

  # sort results
  $sortr = ($sortr) ? ' sort |' : '';
  
  # strict check (exact match)?
  $string = ($virus ne '') ? 
    (($strict) ? 
      "$all_path{'sigtool'}sigtool --list-sigs | $all_path{'grep'}grep $case \"^$virus\$\" | $sortr" :
      "$all_path{'sigtool'}sigtool --list-sigs | $all_path{'grep'}grep $case $virus | $sortr") :
    "$all_path{'sigtool'}sigtool --list-sigs | $sortr";

  # display results
  open (H, $string);
  while (<H>)
  {
    chomp ();
    my $escaped_pure = &urlize (&clamav_vdb_get_pure_name ($_));
    my $escaped = &urlize ($_);
    my $vdb_url = 
      "http://www.rainingfrogs.co.uk/index.php?" .
        "virus=$escaped&search=contains";
    my $symantec_url = 
      "http://search.symantec.com/custom/us/query.html?" .
        "lk=1&rf=0&qt=$escaped_pure&qp=" . &urlize ("url:http://securityresponse.symantec.com/avcenter/venc/data url:/avcenter/venc/data url:/avcenter/venc/auto/index");
    my $mcafee_url = 
      "http://us.mcafee.com/virusInfo/default.asp?" .
        "id=alphar&SearchType=2&searchString=$escaped_pure";
    my $viruspool_url = 
      "http://viruspool.vanderkooij.org/virus.cms?name=$escaped";
    my $viruslist_url = 
      "http://www.viruslist.com/en/find?" .
        "search_mode=virus&words=$escaped_pure";

    if ($first)
    {
      $first = 0;
      
      print qq(
        <table border=1>
        <tr $tb>
	<th>$text{'NAME'}</th>
        <th>Virus Alias Database</th>
        <th>Symantec</th>
        <th>McAfee</th>
        <th>Viruspool</th>
        <th>Viruslist</th>
        </tr>
      );
    }
      
    print qq(
      <tr><td $cb>$_</td>
      <td $cb><a href="$vdb_url" target="_BLANK">$text{'MORE'}</a></td>
      <td $cb><a href="$symantec_url" target="_BLANK">$text{'MORE'}</a></td>
      <td $cb><a href="$mcafee_url" target="_BLANK">$text{'MORE'}</a></td>
      <td $cb><a href="$viruspool_url" target="_BLANK">$text{'MORE'}</a></td>
      <td $cb><a href="$viruslist_url" target="_BLANK">$text{'MORE'}</a></td>
      </tr>
    );
  }
  close (H);
  
  if (!$first)
  {
    print qq(</table>);
  }
  else
  {
    printf "<p>$text{'NO_RESULT'}</p>", $virus;
  }
}

# clamav_get_db_viruses_count ()
# IN: -
# OUT: Number of viruses in ClamAV database
#
# Count the number of viruses in ClamAV databases and return
# it
# 
sub clamav_get_db_viruses_count
{
  my $count = `$all_path{'sigtool'}sigtool --list-sigs | wc -l`;
  
  $count =~ s/ //g;
  
  return $count;
}

# clamav_get_freshclam_daemon_state ()
# IN: -
# OUT: true if the freshclam daemon is deactivated
#
# Return the actual state of the freshclam daemon service
# 
sub clamav_get_freshclam_daemon_state
{
  return ($gconfig{'os_type'} =~ /bsd/) ?
    ($bsd_clam_state eq 'NO') : (&find_byname ('freshclam'));
}

# clamav_bsd_get_state ( $ )
# IN: daemon name (clamav or freshclam)
# OUT: system setting for the daemon (NO or YES)
#
# Return the current system setting for a given daemon for BSD systems
# 
sub clamav_bsd_get_state ( $ )
{
  my $daemon = shift;
  my $ret = 'NO';

  open (H, "</etc/rc.conf");
  while (my $line = <H>)
  {
    if ($line =~ /clamav_${daemon}_enable/)
    {
      chop ($line);
      $ret = (split (/=/, $line))[1];
      $ret =~ s/"//g;
    }
  }
  close (H);

  return $ret;
}


# clamav_verif_refresh_method_ok ()
# IN: -
# OUT: 1 => config say to use a daemon but no daemon exist on the system
#      2 => config say to use a daemon, but a cron exist on the system
#      3 => config say tu use cron, but a daemon exist on the system
#      4 => config and system are ok
#
# Check if the refresh method choose by user in module config is
# ok according to the system
# 
sub clamav_verif_refresh_method_ok
{
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $link = '';
  my $use_cron = 0;
  my $bsd_clam_state = 'NO';
  
  $runlevel = &clamav_get_runlevel ();
  $use_cron = $config{'clamav_refresh_use_cron'};
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  $bsd_clam_state = &clamav_bsd_get_state ('freshclam')
      if ($gconfig{'os_type'} =~ /bsd/);

  $test = `$all_path{'grep'}grep wbmclamav $freshclam`;
  
  if ($use_cron == 0)
  {
    return 1 if (
      ! -e "$freshclam"
    );
    return 2 if (
      -f "/etc/cron.d/clamav-freshclam" ||
      &clamav_get_cron_settings ('update')
    );
  }

  if ($use_cron == 1)
  {
    return 3 if (
      -e "$freshclam" &&
      !$test &&
      ! -f "/etc/cron.d/clamav-freshclam"
    );
  }

  return 4;
}

# clamav_scandir ( $ $ $ $ )
# IN: directory to scan
#     flag to indicate to use or not recursivity
#     flag to indicate to only show infected files or all
#     directory where to put infected files
# OUT: -
#
# Check a given directory and display freshclam output on the current web
# page
# 
sub clamav_scandir ( $ $ $ $ )
{
  my ($dir, $recursive, $infected_only, $move_path) = @_;
  my $report = 0;
  my $infected = 0;
  my $move_path_option = " --move $move_path" if $move_path;
  my $recursive_option = ($recursive) ? ' -r ' : '';

  return if ($move_path =~ /\.\./ || $move_path =~ /;/);
      
  print qq(
    <table border=0 width="90%">
    <tr><td $tb align="center"><b>$text{'CLAMSCAN_COMMAND'}:</b> <code>$all_path{'clamscan'}clamscan $move_path_option $recursive_option $dir</code></td></tr>
    <tr><td valign="top" align="center">
  );
  print qq(<table border=0>);
  open (H, "$all_path{'clamscan'}clamscan $move_path_option $recursive_option $dir 2>&1 |");
  while (<H>)
  {
    /^(.*):(.*)$/;
    my ($file, $state) = ($1, $2);
    my $state_bg = '';
    my $right = '&nbsp;';
    my $is_infected = 0;
    
    next if (!$file);

    if ($state =~ /OK/)
    {
      $state_bg = 'green';
      $state = '&nbsp;';
    }
    elsif ($state =~ /FOUND/)
    {
      $infected++;
      $is_infected = 1;
      $state_bg = 'red';
      $state =~ s/FOUND//g;
      $state = qq(<b>$state</b>);
      if ($move_path)
      {
        $tmp_file = $move_path . basename ($file);
        $right = 
          qq(<input type="checkbox" 
	            name="infected_file$infected" value="$tmp_file">);
      }
      else
      {
        $right = 
          qq(<input type="checkbox" 
	            name="infected_file$infected" value="$file">);
      }
    }
    
    if (($file =~ /known viruses/i) && ($report == 0))
    {
      $report = 1;
      
      print qq(
        <tr><td colspan="3">&nbsp;</td></tr>
        <tr><td colspan="3" align="center">
          <input type="submit" value="$text{'DELETE_SELECTED'}" name="delete">
        </td></tr>
      ) if (&clamav_get_acl ('directories_check_delete') == 1 && $infected > 0);

      print qq(</table>);
      print qq(<table border=0 cellspacing=2 cellpadding=2 width="100%">);
      print qq(<tr><td colspan="3">&nbsp;</td></tr>);
      print qq(<tr><th colspan="3" $cb>Report</th></tr>);
      print qq(<tr><td colspan="3">&nbsp;</td></tr>);
      if ($move_path && $infected)
      {
        print qq(
          <tr><td align="right" width="50%">
	  <b>$text{'INFECTED_FILES_WHERE_MOVED'}</b>:</td>
	  <td colspan="2" align="left" width="50%">$move_path</td></tr>
        );
      }
    }
    
    if (!$report)
    {
      if ((!$is_infected && $infected_only) || $state =~ /moved to/) {}
      else
      {
        $one_infected = 1;
        print qq(<tr><td $cb>$file</td>);

	if (&clamav_get_acl ('directories_check_delete') == 1)
	  {print qq(<td bgcolor="$state_bg">$state</td><td>$right</td></tr>)}
	else
	  {print qq(<td bgcolor="$state_bg">$state</td></tr>)}
      }
    }
    else
    {
      $file =~ tr/a-z/A-Z/;
      $file =~ s/ /_/g;
      print qq(
        <tr><td align="right" width="50%"><b>$text{$file}</b>:</td>
	<td colspan="2" align="left" width="50%">$state</td></tr>
      );
    }
  }
  print qq(</table>) if ($report);
  print qq(</td></tr></table>);
  
  close (H);
}

# clamav_get_last_db_update ()
# IN: -
# OUT: a array with main and daily db update date
#
# Check the signatures databases dates and return them in a
# array
# 
sub clamav_get_last_db_update
{
  my $main = '';
  my $main_infos = '';
  my $daily = '';
  my $daily_infos = '';
         
  if (-e $config{'clamav_db1'})
  {
    $main = &make_date ((stat($config{'clamav_db1'}))[9]);
    open (H, "$all_path{'sigtool'}sigtool --info $config{'clamav_db1'} 2>&1 |");
    while (<H>)
    {
      chomp ();
      my ($name, $value) = split (':');
      $main_infos .= "$text{$name}: <b>$value</b>" if ($name eq 'Version');
      $main_infos .= " - $text{$name}: <b>$value</b>" 
        if ($name eq 'Functionality level');
    }
    close (H);
  }

  if (-e $config{'clamav_db2'})
  {
    $daily = &make_date ((stat($config{'clamav_db2'}))[9]);
    open (H, "$all_path{'sigtool'}sigtool --info $config{'clamav_db2'} 2>&1 |");
    while (<H>)
    {
      chomp ();
      my ($name, $value) = split (':');
      $daily_infos .= "$text{$name}: <b>$value</b>" if ($name eq 'Version');
      $daily_infos .= " - $text{$name}: <b>$value</b>"
        if ($name eq 'Functionality level');
    }
    close (H);
  }

  return ($main, $daily, $main_infos, $daily_infos);
}

# clamav_set_db_no_autoupdate ()
# IN: -
# OUT: -
#
# if user choose to not automatically update db, we stop clamd and
# deactivate cron and services
# 
sub clamav_set_db_no_autoupdate
{
  my $freshclam = '';
  my $freshclambase = '';

  # if a cron file already exist, move it to deactivate it
  if (-f "/etc/cron.d/clamav-freshclam")
  {
    system (
      "$all_path{'mv'}mv /etc/cron.d/clamav-freshclam " .
      " /var/lib/clamav/clamav-freshclam.webmon_NO");
  }

  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);

  if ($gconfig{'os_type'} =~ /bsd/)
  {
    system ("$freshclam stop");
    &clamav_bsd_update_state ('freshclam', 'NO');
  }
  else
  {
    `/usr/bin/perl -pi.bak -e "s/^.*wbmclamav.*\\n\$//" $freshclam`;
    system ("$freshclam stop");
    `/usr/bin/perl -pi.bak -e "s/(\\/bin\\/sh)/\\\$1\\nexit 0 #deactivated by wbmclamav/" $freshclam`;

    if (-f "/var/lib/clamav/interface")
    {
      system (
        "$all_path{'mv'}mv /var/lib/clamav/interface " .
	" /var/lib/clamav/interface.webmin_NO");
    }
  }

  &clamav_delete_cron ('update');
}

# clamav_bsd_update_state ( $ $ )
# IN: daemon name (freshclam or clamav)
#     state (YES or NO)
# OUT: -
#
# Set the system state for a given daemon on BSD systems
# 
sub clamav_bsd_update_state ( $ $ )
{
  my ($daemon, $state) = @_;
  my $found = 0;

  system ("$all_path{'cp'}cp -f /etc/rc.conf /etc/rc.conf_clamav");

  open (SRC, "</etc/rc.conf_clamav");
  open (DST, ">/etc/rc.conf"); 

  while (my $line = <SRC>)
  {
    if ($line =~ /clamav_${daemon}_enable/)
    {
      $line = "clamav_${daemon}_enable=\"$state\"\n";
      $found = 1;
    }

    print DST $line;
  }

  print DST "clamav_${daemon}_enable=\"$state\"\n"
    if (!$found);

  close (DST);
  close (SRC);
}

sub clamav_get_cron_path ()
{
  return (-d "/etc/cron.d/") ? "/etc/cron.d/webmin_clamav" : "/etc/crontab";
}

# clamav_delete_cron ( $ )
# IN: type of the cron (purge|update)
# OUT: -
#
# Delete the line associated with the given type in
# /etc/cron.d/webmin_clamav or /etc/crontab (bsd)
# 
sub clamav_delete_cron ( $ )
{
  my $type = shift;
  my $path = '';
  my $path_sav = '';

  $path = &clamav_get_cron_path ();
  $path_sav = "$path.sav";

  system ("$all_path{'cp'}cp -f $path $path_sav");

  open (SRC, "<$path_sav");
  open (DST, ">$path"); 
  while (my $line = <SRC>)
  {
    print DST $line
      if ($line !~ /^.*#.*$type/);
  }
  close (DST);
  close (SRC);

  unlink ($path_sav);
}

# clamav_set_freshclam_daemon_settings ( $ $ )
# IN: old frequency, new frequency
# OUT: -
#
# Update the check interval frequency of the refresh for the
# freshclam daemon and restart it
# 
sub clamav_set_freshclam_daemon_settings ( $ $ )
{
  my ($oldfreq, $freq) = @_;
  my $freshclam = '';
  my $freshclambase = '';

  # if a cron file already exist, move it to deactivate it
  if (-f "/etc/cron.d/clamav-freshclam")
  {
    system (
      "$all_path{'mv'}mv /etc/cron.d/clamav-freshclam " .
      " /var/lib/clamav/clamav-freshclam.webmon_NO");
  }

  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
 
 if ($gconfig{'os_type'} =~ /bsd/)
  {
    &clamav_bsd_update_state ('freshclam', 'YES');
    system ("$config{'clamav_freshclam_init_script'} start");
  }
  else
  {
    `/usr/bin/perl -pi.bak -e "s/^.*wbmclamav.*\\n\$//" $freshclam`;

    if ($gconfig{'os_type'} =~ /debian/ && 
      -f "/var/lib/clamav/interface.webmin_NO")
      {
        system (
	  "$all_path{'mv'}mv /var/lib/clamav/interface.webmin_NO " .
	  " /var/lib/clamav/interface");
      }
  }
 
  &clamav_load_freshclam_config ();
  $freshclam_config{'Checks'} = $freq;
  &clamav_save_freshclam_config ();

  system ("$freshclam restart");
}

# clamav_save_freshclam_config ()
# IN: -
# OUT: -
#
# Save the freshclam configuration in freshclam config file
# 
sub clamav_save_freshclam_config
{
  my @tmp = ();
  
  &lock_file ($config{'clamav_freshclam_conf'});
  open (H, ">$config{'clamav_freshclam_conf'}");
  foreach my $key (keys %freshclam_config)
  {
    next if $freshclam_config{$key} eq "$text{'UNDEFINED'}";
    next if ($freshclam_config{$key} eq '' && 
      $freshclam_predefined{$key} == 1);
   
    my @tmp = split (' ', $freshclam_config{$key});
    if ($#tmp > 0)
      {foreach (@tmp) {print H "$key $_\n"}}
    else
      {print H "$key $freshclam_config{$key}\n"};
  }
  close (H);
  &unlock_file ($config{'clamav_freshclam_conf'});
}

# clamav_save_global_settings ()
# IN: -
# OUT: -
#
# Save ClamAV/Freshclam settings in config files
# 
sub clamav_save_global_settings
{
  my $restart = shift;
  my $cc = '';
  my $fc = '';
  
  $cc = $config{'clamav_clamav_conf'};
  $fc = $config{'clamav_freshclam_conf'};

  if (!$restart)
  {
    $cc = "$config{'clamav_working_path'}/.clamav/clamav.conf";
    $fc = "$config{'clamav_working_path'}/.clamav/freshclam.conf";
  }
  
  &lock_file ($cc); open (CLAMAV, ">$cc");
  &lock_file ($fc); open (FRESHCLAM, ">$fc");
  
  foreach $key (sort keys %in)
  {
    $value = $key;

    next if ($restart && $in{$key} eq "$text{'UNDEFINED'}");
    
    if ($key =~ /^clamav.*$/)
    {
      $value =~ s/^clamav_//;
      next if ($restart && ($in{$key} eq '' && 
        $clamav_predefined{$value} == 1));
      print CLAMAV "$value $in{$key}\n";
    }
    elsif ($key =~ /^freshclam.*$/)
    {
      $value =~ s/^freshclam_//;
      next if ($restart && ($in{$key} eq '' && 
        $freshclam_predefined{$value} == 1));
	
      my @tmp = split (' ', $in{$key});
      if ($#tmp > 0)
        {foreach (@tmp) {print FRESHCLAM "$value $_\n"}}
      else
        {print FRESHCLAM "$value $in{$key}\n"};
    }
  }
  
  close (CLAMAV); &unlock_file ($cc);
  close (FRESHCLAM); &unlock_file ($fc);

  if ($restart)
  {
    &clamav_activate_clamd ();
    
    system ("$config{'clamav_freshclam_init_script'} restart")
      if ($config{'clamav_refresh_use_cron'} eq '0');

    &clamav_clean_global_settings_tempfiles ();
  }
}

# clamav_global_settings_get_delete_item ()
# IN: type of item to process ('clamav' or 'freshclam')
# OUT: keys of the item to delete (clamav predefine variable name)
#
# Return the name of the predefined variable to delete in clamav or
# freshclam's config file
# 
sub clamav_global_settings_get_delete_item
{
  my $type = shift;
  my $exp = '';
  my $key = '';

  $exp = 'ns' . $type . '_delete_';

  foreach $key (keys %in)
  {
    if ($key =~ /^$exp/)
    {
      $key =~ s/^$exp//g;
      
      return $key;
    }
  }

  return $key;
}

# clamav_clean_global_settings_tempfiles ()
# IN: -
# OUT: -
#
# Remove temporary files for global settings section
# 
sub clamav_clean_global_settings_tempfiles
{
  unlink ("$config{'clamav_working_path'}/.clamav/clamav.conf");
  unlink ("$config{'clamav_working_path'}/.clamav/freshclam.conf");
}

# clamav_display_clamav_settings ()
# IN: -
# OUT: -
#
# Load clamav configuration file and display it
# 
sub clamav_display_clamav_settings
{
  my ($newkey, $deletekey) = @_;
  my $key = '';
  
  &clamav_load_clamav_config ();
  
  if ($deletekey || $newkey)
  {
    # delete the specified key if it has been specified
    delete $in{"clamav_$deletekey"} if ($deletekey);

    # add a new key in the config file
    $in{"clamav_$newkey"} = 
      ($clamav_predefined{$newkey} == 1) ? "$text{'UNDEFINED'}" : '' 
        if ($newkey);
    
    # save and reload config
    &clamav_save_global_settings (0);
    &clamav_load_clamav_config ();
  }
  
  if (&clamav_get_acl ('global_settings_write') == 1 &&
    &clamav_display_combo_clamav_predefined (1) == 1)
  {
    print qq( 
      <input type="submit" name="nsclamav_add" value="$text{'ADD_KEY'}">
    );
  }
  
  print qq(<table border=1>);
  foreach $key (sort keys %clamav_config)
  {
    next if ($key eq '');
   
    if ($clamav_config{$key} eq '')
    {
      print qq(
        <tr><td $cb>$key</td>
        <input type="hidden" name="clamav_$key">
	<td><font color="silver"><i>$text{'NO_VALUE'}</i></font></td>
      );
    }
    elsif ($clamav_config{$key} eq "$text{'UNDEFINED'}")
    {
      print qq(
        <tr><td bgcolor="gray"><font color="white"><b>$key</b></font></td>
        <td><input type="text" name="clamav_$key" size=40
        value="$clamav_config{$key}"></td>
      );
    }
    else
    {
      print qq(
        <tr><td $cb>$key</td>
        <td><input type="text" name="clamav_$key" size=40
        value="$clamav_config{$key}"></td>
      );
    }
    if (&clamav_get_acl ('global_settings_write') == 1)
    {
      print qq(
        <td><input title="$text{'DELETE_ITEM'}" type="submit"
        name="nsclamav_delete_$key"
        value="$text{'DELETE'}"></td>
        </tr>
      );
    }
  }
  print qq(</table>);
}

# clamav_display_freshclam_settings ()
# IN: -
# OUT: -
#
# Load freshclam configuration file and display it
# 
sub clamav_display_freshclam_settings
{
  my ($newkey, $deletekey) = @_;
  my $key = '';
  my %tmp_config = ();
  
  &clamav_load_freshclam_config ();
  
  if ($deletekey || $newkey)
  {
    # delete the specified key if it has been specified
    delete $in{"freshclam_$deletekey"} if ($deletekey);

    # add a new key in the config file
    $in{"freshclam_$newkey"} = 
      ($freshclam_predefined{$newkey} == 1) ? "$text{'UNDEFINED'}" : '' 
        if ($newkey);
    
    # save and reload config
    &clamav_save_global_settings (0);
    &clamav_load_freshclam_config ();
  }
  
  if (&clamav_get_acl ('global_settings_write') == 1 &&
    &clamav_display_combo_freshclam_predefined (1) == 1)
  {
    print qq( 
      <input type="submit" name="nsfreshclam_add" value="$text{'ADD_KEY'}">
    );
  }
 
  print qq(<table border=1>);
  foreach $key (sort keys %freshclam_config)
  {
    next if ($key eq '');
   
    if ($freshclam_config{$key} eq '')
    {
      print qq(
        <tr><td $cb>$key</td>
        <input type="hidden" name="freshclam_$key">
	<td><font color="silver"><i>$text{'NO_VALUE'}</i></font></td>
      );
    }
    elsif ($freshclam_config{$key} eq "$text{'UNDEFINED'}")
    {
      print qq(
        <tr><td bgcolor="gray"><font color="white"><b>$key</b></font></td>
        <td><input type="text" name="freshclam_$key" size=40
        value="$freshclam_config{$key}"></td>
      );
    }
    else
    {
      print qq(
        <tr><td $cb>$key</td>
        <td><input type="text" name="freshclam_$key" size=40
        value="$freshclam_config{$key}"></td>
      );
    }
    if (&clamav_get_acl ('global_settings_write') == 1)
    {
      print qq(
        <td><input title="$text{'DELETE_ITEM'}" type="submit"
        name="nsfreshclam_delete_$key"
        value="$text{'DELETE'}"></td>
        </tr>
      );
    }
  }
  print qq(</table>);
}

# clamav_load_clamav_config ()
# IN: -
# OUT: -
#
# Load the clamav config file in memory
# 
sub clamav_load_clamav_config
{
  my $cc = '';
  
  %clamav_config = undef;
  
  $cc = $config{'clamav_clamav_conf'};
  $cc = "$config{'clamav_working_path'}/.clamav/clamav.conf" if (-f "$config{'clamav_working_path'}/.clamav/clamav.conf");
  
  open (H, "<$cc");
  while (<H>)
  {
    next if /#/;

    (my $key, my @value) = split (' ');
    $clamav_config{$key} = join (' ', @value);
  }
  close (H);
}

# clamav_load_freshclam_config ()
# IN: -
# OUT: -
#
# Load the freshclam config file in memory
# 
sub clamav_load_freshclam_config
{
  my $fc = '';
  
  %freshclam_config = undef;
  
  $fc = $config{'clamav_freshclam_conf'};
  $fc = "$config{'clamav_working_path'}/.clamav/freshclam.conf" if (-f "$config{'clamav_working_path'}/.clamav/freshclam.conf");
  
  open (H, "<$fc");
  while (<H>)
  {
    next if /#/;

    (my $key, my @value) = split (' ');
    
    if (!defined $freshclam_config{$key})
      {$freshclam_config{$key} = join (' ', @value)}
    else
      {$freshclam_config{$key} .= ' ' . join (' ', @value)}
  }
  close (H);
}

# clamav_set_cron_update ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set the cron line for the db refresh in /etc/cron.d/webmin_clamav
# 
sub clamav_set_cron_update ( $ $ )
{
  my ($hour, $day) = @_;
  my $new_line = '';
  my $datadir = '';
  my $path = '';

  $path = &clamav_get_cron_path ();

  $datadir = dirname ($config{'clamav_db1'});

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('update');
  $new_line = 
    "0 $hour * * $day root \$($all_path{'freshclam'}freshclam " .
    "--quiet " .
    "--config-file $config{'clamav_freshclam_conf'} " .
    "--log $config{'clamav_freshclam_log'} " .
    "--datadir $datadir) " .
    "#update";

  open (H, ">>$path");
  print H "$new_line\n";
  close (H);
}

# clamav_get_logfiles ()
# IN: -
# OUT: A array with logfiles
#
# Identify ClamAV log files and fill a array with them
#
sub clamav_get_logfiles
{
  @ret = ();

  foreach $key (%config)
  {
    push @ret, $key if ($key =~ /^.*\.log$/);
  }

  return @ret;
}

# clamav_set_cron_purge ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set cron line for the quarantine repository purge
# 
sub clamav_set_cron_purge ( $ $ )
{
  my ($hour, $day, $maxdays) = @_;
  my $new_line = '';
  my $path = '';

  $path = &clamav_get_cron_path ();

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('purge');

  $new_line =
    "0 $hour * * $day root \$($root_directory/clamav/bin/purge_quarantine " .
    " $config{'clamav_quarantine'} " .
    " $maxdays" .
    ") #purge";

  open (H, ">>$path");
  print H "$new_line\n";
  close (H);
}

# clamav_update_db ()
# IN: -
# OUT: -
#
# Update virus signatures databases displaying output on the current web
# page
# 
sub clamav_update_db
{
  print qq(<div><pre style="background: #cccccc">);;
  open (H, "$all_path{'freshclam'}freshclam 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    if (/load/) {print '.'} else {print};
  }
  close (H);
  print qq(</pre>);
}

# clamav_get_cron_settings ( $ )
# IN: type of the cron (purge|update)
# OUT: a array representing a cron line
#
# Return a array of a cron line corresponding to the given type. if line
# is not found, return undef
# 
sub clamav_get_cron_settings ( $ )
{
  my $type = shift;
  my @lines = ();
  my @cron_line = ();
  my @tmp = ();
  my $path = '';

  $path = &clamav_get_cron_path ();

  return undef if (! -e "$path");

  open (H, "<$path");
  @lines = <H>;
  close (H);
                                                                                
  foreach (@lines)
  {
    @tmp = split (/#/, $_);
    $tmp[0] =~ s/[\$,(,)]+//g;
    @cron_line = split (/\s+/, $tmp[0]);

    return @cron_line
      if ($tmp[1] =~ /$type/);
  }
                                                                                
  return undef;
}

# clamav_freshclam_daemon_settings_table ( $ )
# IN: frequency
# OUT: a buffer to display
#
# Build a HTML table with all possible frequencies for freshclam
# daemon
# 
sub clamav_freshclam_daemon_settings_table ( $ )
{
  my $freq = shift;

  $buffer .= qq(
    <table border="1">
    <tr $tb><td><b>$text{'FREQUENCY'}</b></td></tr>
    <tr $cb>
    <td><select name="freq">
  );

  foreach (1..50)
  {
    $default = ($_ == $freq) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

  $buffer .= qq(</select></td></tr></table>);

  return $buffer;
}

# clamav_cron_settings_table ( $ $ )
# IN: default cron hour, default cron day
# OUT: a buffer to display
#
# Build a HTML table for choose hour a day for execution of refresh
# db cron
#
sub clamav_cron_settings_table
{
  my ($hour, $day) = @_;
  my $buffer = '';
  my $default = '';
  my $every_hour = '';

  ($every_hour, $hour) = split (/\//, $hour);
  ($every_hour, $hour) = ($hour, $every_hour) if (!$hour);
  
  $default = ($every_hour) ? ' CHECKED' : '';
  $buffer .= qq(
    <table border="1">
    <tr $tb><td><b>$text{'HOUR'}</b></td><td><b>$text{'DAY'}</b></td></tr>
    <tr $cb><td valign="bottom">
    <small><i><input id="every_hours" type="checkbox" name="every_hours"$default>&nbsp;<label for="every_hours">$text{'EVERY_X_HOURS'}</label></i></small><br><select name="hour">
  );

  foreach (0..23)
  {
    $default = ($_ == $hour) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

  $buffer .= qq(</select></td><td valign="bottom"><select name="day">);

  $day = 7 if ($day eq '*');
  $default = ($day == 7) ? ' SELECTED' : '';
  $buffer .= qq(<option value="7"$default>$text{'EVERYDAY'}</option>\n);
  for (my $i = 0; $i < 7; $i++)
  {
    my $value = $days[$i];
    my $default = ($i == $day) ? ' SELECTED' : '';
                                                                                
    $buffer .= qq(<option value="$i"$default>$value</option>\n);
  }

  $buffer .= qq(</select></td></tr></table>);

  return $buffer;
}

# clamav_is_quarantine_repository_empty ()
# IN: -
# OUT: true if empty
#
# Check wether the quarantine repository is empty or not.
# 
sub clamav_is_quarantine_repository_empty
{
  my @files = ();

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir(DIR);
  closedir (DIR);

  return (($#files - 1) <= 0);
}

# clamav_cut_string ( $ $ )
# IN: string, max length
# OUT: a cutted string
#
# Cut a string to fit in the given length, adding "[..]" at
# the end
# 
sub clamav_cut_string ( $ $ )
{   
  my ($string, $max_len) = @_;

  $string = (substr ($string, 0, $max_len - 5)) . ' [..]'
    if (($max_len > 5) && (length ($string) >= $max_len));
                                                                                
  return $string;
}

# clamav_is_clamd_installed ()
# IN: -
# OUT: -
#
# Check if specified clamd path is ok on the system
# 
sub clamav_is_clamd_installed
{
  return (-e $config{'clamav_init_script'});
}

# clamav_activate_clamd ()
# IN: -
# OUT: -
#
# Restart the clamav daemon, waiting on second for process
# to be up
# 
sub clamav_activate_clamd
{
  system ("$config{'clamav_init_script'} restart");
  sleep (1);
}

# clamav_deactivate_clamd ()
# IN: -
# OUT: -
#
# Stop the clamav daemon, waiting one second for process
# to be down
# 
sub clamav_deactivate_clamd
{
  system ("$config{'clamav_init_script'} stop");
  sleep (1);
}

# clamav_is_clamd_alive ()
# IN: -
# OUT: -
#
# Check wether or not clamd daemon is alive
# 
sub clamav_is_clamd_alive
{
  return (&clamav_is_milter ()) ?
    &find_byname ('clamav-milter') : &find_byname ('clamd');
}

# clamav_get_email_header_values ( $ $ )
# IN: email file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
# 
sub clamav_get_email_header_values ( $ @ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;

  if ($file =~ /\.gz$/)
  {
    my $gzip = undef;
    
    $gzip = gzopen ($file, "r");
    LOOP1: while ($gzip->gzreadline ($item) && ($i <= $#names))
    {
      chomp ($item);
      $item =~ s/^H\?\?//;
      $item =~ s/^[ ,\t]*//g;
      
      foreach my $name (@names)
      {
        if ($item =~ /^$name: /i)
        {
          $i++ if (!defined $header{$name});
          $item =~ s/^$name: //i;
          $header{$name} = $item;
	  next LOOP1;
        }
      }
    }

    $gzip->gzclose ();
  }
  elsif (open (H, $file))
  {
    LOOP2: while (defined ($item = <H>) && ($i <= $#names))
    {
      chomp ($item);
      $item =~ s/^H\?\?//;
      $item =~ s/^[ ,\t]*//g;

      foreach my $name (@names)
      {
        if ($item =~ /^$name: /i)
        {
          $i++ if (!defined $header{$name});
          $item =~ s/^$name: //i;
          $header{$name} = $item;
	  next LOOP2;
        }
      }
    }

    close (H);
  }
                                                                                
  return %header;
}

# clamav_get_log_values ( $ $ )
# IN: log file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
#
sub clamav_get_log_values ( $ $ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;
                                                                                
  if (open (H, $file))
  {
    LOOP1: while (defined ($item = <H>) && ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /: $name: .*\n$/i)
        {
          $i++ if (!defined $header{$name});
          $item =~ s/^.*: $name:.*: //i;
          $header{$name} = $item;
	  next LOOP1;
        }
      }
    }
    close (H);
  }
                                                                                
  return %header;
}

# clamav_print_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Display the content of the given mail file
# 
sub clamav_print_email ( $ )
{
  my $file = shift;

  exit (1) if ($file =~ /\.\./ || $file =~ /;/);
    
  # is amavisd-new installed or amavis-ng
  $file = (&clamav_is_amavis_ng ()) ? "$file.msg" : "$file";

  if ($file =~ /\.gz$/)
  {
    my $gzip = undef;
    my $content = '';

    $gzip = gzopen ("$config{'clamav_quarantine'}/$file", "r");
    $gzip->gzread ($content);
    $gzip->gzclose ();
    print qq(<textarea cols=80 rows=30>$content</textarea>\n);
  }
  else
  {
    my @content = ();
    my @body = ();
    
    open (H, "$config{'clamav_quarantine'}/$file");
    @content = <H>;
    close (H);

    # header and body are in separate files for MailScanner
    if (&clamav_is_mailscanner ())
    {
      my $dir = dirname ($file);
      my $name = basename ($file);

      $name =~ s/^./d/;

      open (H, "$config{'clamav_quarantine'}/$dir/$name");
      @body = <H>;
      close (H);
    }
    
    print qq(<textarea cols=80 rows=30>@content\n@body</textarea>\n);
  }
}

# clamav_print_email_infos ( $ )
# IN: email file on the disk
# OUT: -
#
# Display some email information in a HTML table
# 
sub clamav_print_email_infos ( $ )
{
  my $base = shift;
  my %header = ();
  my $subject = '';
  my $from = '';
  my $to = '';

  # if amavis-ng is installed
  if (&clamav_is_amavis_ng ())
  {
    %header = &clamav_get_email_header_values (
      "$config{'clamav_quarantine'}/$base.msg",
      qw(Subject X-Quarantined-From X-Quarantined-To)
    );
    $subject = $header{'Subject'};
    $from = $header{'X-Quarantined-From'};
    $to = $header{'X-Quarantined-To'};
   }
  # if amavisd-new or clamav-milter are installed
  else
  {
    %header = &clamav_get_email_header_values (
      "$config{'clamav_quarantine'}/$base",
      qw(Subject From To)
    );
    $subject = $header{'Subject'};
    $from = $header{'From'};
    $to = $header{'To'};
  }

  $subject = &clamav_html_encode ($subject);
  $from = &clamav_html_encode ($from);
  $to = &clamav_html_encode ($to);

  print qq(
    <p><table border=0>
    <tr><td $cb><b>$text{'SUBJECT'}: </b></td><td>$subject</td><tr>
    <tr><td $cb><b>$text{'FROM'}: </b></td><td>$from</td><tr>
    <tr><td $cb><b>$text{'TO'}: </b></td><td>$to</td><tr>
    </table></p>
  );
}

# clamav_resend_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Resend a false positive email. it first test if there is an
# existing amavis-inject script on the server. if not, it use
# it is proper copy of this perl script (./bin/amavis-inject).
#
# supported MTA are:
#
#   - rsmtp
#   - sendmail
# 
sub clamav_resend_email ( $ )
{
  my $base = shift;
  my $path = $config{'clamav_quarantine'};
  my $amavis = "$all_path{'amavis-inject'}amavis-inject ";
  my $mta = " $all_path{'sendmail'}sendmail -bs "; # use sendmail by default
  my $args = '';

  return if ($base =~ /\.\./ || $base =~ /;/);

  %header = &clamav_get_email_header_values ("$path/$base", qw(From To));
  if ($header{'From'} =~ /\<(.*)\>/) {$header{'From'} = $1}
  if ($header{'To'} =~ /\<(.*)\>/) {$header{'To'} = $1}
  $args = " -s $header{'From'} -r $header{'To'} ";
  
  # if rsmtp is found, use it instead of sendmail
  $mta = " $all_path{'rsmtp'}rsmtp " if ($all_path{'rsmtp'} ne '');
  # if amavis-inject not found, use our own copy  
  $amavis = "./bin/amavis-inject " if ($all_path{'amavis-inject'} eq '');

  # if amavis-ng is not installed
  if (!&clamav_is_amavis_ng ())
  {
      ($base =~ /\.gz$/) ?
        system ("$all_path{'zcat'}zcat $path/$base | " .
	        "$amavis $args | $mta") :
	system ("$amavis $args $path/$base | $mta");
  }
  # if amavis-ng is installed
  else
  {
      ($base =~ /\.gz$/) ?
      system ("$all_path{'zcat'}zcat $path/$base | " .
              "$amavis $args | $mta") :
      system ("$amavis $args $path/$base.msg | $mta")
  }
}

# clamav_remove_email ( $ )
# IN: base name of the email file on the disk
# OUT: -
#
# Delete a email and its associated log file on the quarantine
# repository
# 
sub clamav_remove_email
{
  my $base = shift;

  exit (1) if ($base =~ /\.\./ || $base =~ /;/);
  
  # if amavis-ng is installed
  if (&clamav_is_amavis_ng ())
  {
    system("$all_path{'rm'}rm -f $config{'clamav_quarantine'}/$base.*");
  }
  # if mailscanner 
  elsif (&clamav_is_mailscanner ())
  {
    system("$all_path{'rm'}rm -rf $config{'clamav_quarantine'}/" . 
      dirname ($base));
  }
  # if amavisd-new or clamav-milter is installed
  else
  {
    system("$all_path{'rm'}rm -f $config{'clamav_quarantine'}/$base");
  }
}

# clamav_print_log ( $ $ )
# IN: file to display, number of lines to display
# OUT: -
#
# Display the content of the given log file
#
sub clamav_print_log
{
  my $file = shift;
  my $lines = shift;
  my @content = ();

  exit (1) if ($file =~ /;/ || $lines =~ /;/);
  
  open (H, "$file");
  if (!$lines)
    {@content = <H>}
  else
    {@content = `/usr/bin/tail -n $lines $file`}
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_quarantine_print_log ( $ )
# IN: email log file on the disk
# OUT: -
#
# Display the content of the given mail log file
#
sub clamav_quarantine_print_log ( $ )
{
  my $file = shift;
  my @content = ();

  exit (1) if ($file =~ /\// || $file =~ /\.\./ || $file =~ /;/);
    
  open (H, "$config{'clamav_quarantine'}/$file.log");
  @content = <H>;
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_print_quarantine_table_mailscanner ( $ $ $ $ $ $ $ $ $ $ $ )
# IN: search type ('virus', 'spam')
#     current page
#     virus name to search for
#     from
#     to
#     day1
#     month1
#     year1
#     day2
#     month2
#     year2
# OUT: -
#
# -- used if mailscanner has been found --
# 
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_mailscanner ( $ $ $ $ $ $ $ $ $ $ $)
{
  my ($search_type, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my $date1 = '';
  my $date2 = '';
  my $i = 0;
  my $count = 0;
  my @files = ();
  my @arows = ();
  my $cyear = &clamav_get_current_year ();
  my $is_spam = 0;

  $current = int ($current);

  $date1 = &clamav_format_date ($day1, $month1, ($year1) ? $year1 : $cyear);
  $date2 = &clamav_format_date ($day2, $month2, ($year2) ? $year2 : $cyear);

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  while (my $dir = readdir (DIR))
  {
    next if ($dir =~ /^\./);
    
    opendir (DIR1, "$config{'clamav_quarantine'}/$dir/");
    while (my $dir1 = readdir (DIR1))
    {
      next if ($dir1 =~ /^\./) || ($dir1 eq 'spam');

      $is_spam = (open (SPAM, 
        "$config{'clamav_quarantine'}/$dir/spam/df$dir1"));
      close (SMAP) if ($is_spam);

      opendir (DIR2, "$config{'clamav_quarantine'}/$dir/$dir1/");
      while (my $file = readdir (DIR2))
      {
        next if ($file =~ /^\./);
        next if (($search_type eq 'virus' || $search_type eq 'spam') && 
	  $file !~ /^q/ && $file ne 'message');
	
	$file .= '-spam' if $is_spam;
        push @files, "$dir/$dir1/$file";
      };
      closedir (DIR2);
    };
    closedir (DIR1);
  }

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i++)
  {
    my $msg = $files[$i];
    my $quarantine_path = $config{'clamav_quarantine'};

    # virus type
    if ($search_type eq 'virus' && $msg !~ /-spam$/)
    {
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$msg",
          qw(Subject From To)
        );

      next if (!&clamav_quarantine_add_row ($header{'Subject'}, '', 
        $msg, '...', '...', $mail_from, $header{'From'}, $mail_to, 
        $header{'To'}, "$quarantine_path/$msg", $date1, $date2, \@arows));

        $count++;
    }
    # spam type
    elsif ($search_type eq 'spam' && $msg =~ /-spam$/)
    {
      $msg =~ s/-spam$//;
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$msg",
          qw(Subject From To)
        );

      next if (!&clamav_quarantine_add_row ($header{'Subject'}, '',
        $msg, '', '', $mail_from, $header{'From'}, $mail_to,
        $header{'To'}, "$quarantine_path/$msg", $date1, $date2, \@arows));
        $count++;
    }
  }
  
  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  @arows = sort clamav_sort_by_date_quarantine_table (@arows);

  &clamav_print_quarantine_table_display ($current, $search_type, 
    \%hrow, \@arows);

  return $count;
}

# clamav_get_current_year ()
# IN: -
# OUT: -
#
# return the current year
# 
sub clamav_get_current_year ()
{
  return ((localtime (time ()))[5]) + 1900;
}

# clamav_print_quarantine_table_milter ( $ $ $ $ $ $ $ $ $ $ $ )
# IN: search type ('virus', 'spam')
#     current page
#     virus name to search for
#     from
#     to
#     day1
#     month1
#     year1
#     day2
#     month2
#     year2
# OUT: -
#
# -- used if clamav-milter has been found --
# 
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_milter ( $ $ $ $ $ $ $ $ $ $ $ )
{
  my ($search_type, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my @files = ();
  my @arows = ();
  my $i = 0;
  my $count = 0;
  my $date1 = '';
  my $date2 = '';
  my $cyear = &clamav_get_current_year ();

  $current = int ($current);

  $date1 = &clamav_format_date ($day1, $month1, ($year1) ? $year1 : $cyear);
  $date2 = &clamav_format_date ($day2, $month2, ($year2) ? $year2 : $cyear);

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  while (my $dir = readdir (DIR))
  {
    next if ($dir =~ /^\./);
    
    opendir (DIR1, "$config{'clamav_quarantine'}/$dir/");
    while (my $file = readdir (DIR1))
    {
      next if ($file =~ /^\./);
      push @files, "$dir/$file";
    };
    closedir (DIR1);
  }

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i++)
  {
    next if ($files[$i] =~ /^\./) || ($files[$i] =~ /^spam/);
      
    my $msg = $files[$i];
    my $quarantine_path = $config{'clamav_quarantine'};
    my %header = &clamav_get_email_header_values (
        "$quarantine_path/$msg",
        qw(Subject From To)
      );
    my $virus = '';
    my $url_virus = '';
    
    $msg =~ /msg\.[\w]+\.(.*)$/;
    $virus = $1;
    $url_virus = &urlize ($virus);

    next if (!&clamav_quarantine_add_row ($header{'Subject'}, $url_virus, 
      $msg, $virus_name, $virus, $mail_from, $header{'From'}, $mail_to, 
      $header{'To'}, "$quarantine_path/$msg", $date1, $date2, \@arows));

      $count++;
  }
  
  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  @arows = sort clamav_sort_by_date_quarantine_table (@arows);

  &clamav_print_quarantine_table_display ($current, $search_type, 
    \%hrow, \@arows);

  return $count;
}

# clamav_print_quarantine_table_amavis_ng ( $ $ $ $ $ $ $ $ $ $ $ )
# IN: search type ('virus', 'spam')
#     current page
#     virus name to search for
#     from
#     to
#     day1
#     month1
#     year1
#     day2
#     month2
#     year2
# OUT: -
#
# -- used if amavis-ng has been found --
# 
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_amavis_ng ( $ $ $ $ $ $ $ $ $ $ $ )
{
  my ($search_type, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my $STRING_MAX_LEN = 30;
  my $date1 = '';
  my $date2 = '';
  my $i = 0;
  my $page_count = 0;
  my $count = 0;
  my @files = ();
  my @arows = ();
  my $cyear = &clamav_get_current_year ();

  $current = int ($current);

  $date1 = &clamav_format_date ($day1, $month1, ($year1) ? $year1 : $cyear);
  $date2 = &clamav_format_date ($day2, $month2, ($year2) ? $year2 : $cyear);

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i += 2)
  {
    next if ($files[$i] eq '.');
      
    my $log = $files[$i];
    my $msg = $files[$i + 1];
    my $quarantine_path = $config{'clamav_quarantine'};
    my $base = $log; $base =~ s/\..*$//g;
    my %header = &clamav_get_email_header_values (
        "$quarantine_path/$base.msg",
        qw(Subject X-Quarantined-From X-Quarantined-To)
      );
    my $subject = $header{'Subject'};
    my $from = $header{'X-Quarantined-From'};
    my $to = $header{'X-Quarantined-To'};
    my %header = &clamav_get_log_values (
        "$quarantine_path/$base.log",
        qw(AMAVIS::AV::CLAMD)
      );
    my $virus = $header{'AMAVIS::AV::CLAMD'};
    my $url_virus = &urlize ($virus);
    my $fdate = '';

    next if (!&clamav_quarantine_add_row ($subject, $url_virus, 
      $base, $virus_name, $virus, $mail_from, $from, $mail_to, $to, 
      "$quarantine_path/$base.msg", $date1, $date2, \@arows));

      $count++;
  }
  
  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  print qq(
    <table>
    <tr $tb>
    <td><b>$text{'DATE'}</b></td>
    <td><b>$text{'SUBJECT'}</b></td>
    <td><b>$text{'VIRUS'}</b></td>
    <td><b>$text{'FROM'}</b></td><td><b>$text{'TO'}</b></td>);
    
  if (&clamav_get_acl ('quarantine_resend') +
    &clamav_get_acl ('quarantine_delete') != 0)
    {print qq(<td colspan=2><b>$text{'ACTION'}</b></td>)}
    
  print qq(</tr>);

  # write javascript function to select/unselect all items
  &clamav_print_javascript_check_uncheck_all (1, 'quarantine_file');

  # display search result
  for ($i = ceil ($current * $PAGE_ITEMS); 
    $i < $#arows + 1 && $page_count++ < $PAGE_ITEMS; $i++)
  {
    my %hrow = %{$arows[$i]};

    my $subjectc = &clamav_cut_string ($hrow{'subject'}, $STRING_MAX_LEN);
    my $fromc = &html_escape (&clamav_cut_string ($hrow{'from'}, 
      $STRING_MAX_LEN));
    my $toc = &html_escape (&clamav_cut_string ($hrow{'to'}, $STRING_MAX_LEN));
    my $virusc = &clamav_cut_string ($hrow{'virus'}, $STRING_MAX_LEN);
     
    print qq(
      <tr>
      <td $cb title="$text{'DATE'}: $hrow{'date'}\n$text{'FROM'}: $hrow{'from'}\n$text{'TO'}: $hrow{'to'}\n$text{'VIRUS'}: $hrow{'virus'}\n$text{'SUBJECT'}: $hrow{'subject'}"><i><small><a href="quarantine_viewmail.cgi?base=$hrow{'base'}">$hrow{'date'}</a></small></i></td>
      <td $cb>$subjectc</td>
      <td $cb><b><a href="/clamav/vdb_search_main.cgi?search=on&virus=$hrow{'url_virus'}" title="$text{'VIRUS'}: $hrow{'virus'}">$virusc</a></b></td>
      <td $cb title="$text{'FROM'}: $hrow{'from'}">$fromc</td>
      <td $cb title="$text{'TO'}: $hrow{'to'}">$toc</td>
      <td $cb><a href="quarantine_viewlog.cgi?base=$hrow{'base'}">
      $text{'VIEWLOG'}</a></td>);

    if (&clamav_get_acl ('quarantine_resend') +
      &clamav_get_acl ('quarantine_delete') != 0)
    {
      if (&clamav_get_acl ('quarantine_resend') == 1)
        {print qq(<td $cb><a href="quarantine_resend.cgi?base=$hrow{'base'}">$text{'RESEND'}</a></td>)}
      else
        {print qq(<td></td>)}
	    
      if (&clamav_get_acl ('quarantine_delete') == 1)
        {print qq(<td $cb><input type="checkbox" name="quarantine_file$i" value="$hrow{'base'}"></td>)}
      else
        {print qq(<td></td>)}
    }
	
    print qq(</tr>);
  }

  print qq(</table>);

  if (&clamav_get_acl ('quarantine_delete') == 1)
  {
    print qq(
      <p><input id="checkuncheck" type="checkbox" onClick="check_uncheck_all (this.checked)"> <label for="checkuncheck">$text{'CHECK_UNCHECK_ALL'}</label></p>
      <p><input type="submit" name="delete" value="$text{'DELETE_SELECTED'}"> <input type="submit" name="delete_all" value="$text{'PURGE_QUARANTINE_NOW'}"></p>);
  }

  return $count;
}

# clamav_print_quarantine_table_amavisd_new ( $ $ $ $ $ $ $ $ $ $ $ $)
# IN: search type ('virus', 'spam')
#     current page
#     virus name to search for
#     from
#     to
#     day1
#     month1
#     year1
#     day2
#     month2
#     year2
# OUT: -
# 
# -- used if amavisd-new has been found --
#
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table_amavisd_new ( $ $ $ $ $ $ $ $ $ $ $ )
{
  my ($search_type, $current, $virus_name, $mail_from, $mail_to,
      $day1, $month1, $year1,
      $day2, $month2, $year2) = @_;
  my $date1 = '';
  my $date2 = '';
  my $i = 0;
  my $count = 0;
  my @files = ();
  my @arows = ();
  my $cyear = &clamav_get_current_year ();

  $current = int ($current);

  $date1 = &clamav_format_date ($day1, $month1, ($year1) ? $year1 : $cyear);
  $date2 = &clamav_format_date ($day2, $month2, ($year2) ? $year2 : $cyear);

  $virus_name =~ s/ //g;
  $mail_from =~ s/ //g;
  $mail_to =~ s/ //g;

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);

  # browse result and push it in a array to manage
  # pagination
  $count = 0;
  for ($i = 0; $i < $#files + 1; $i++)
  {
    # virus type
    if ($search_type eq 'virus' && $files[$i] =~ /^virus/)
    {
      my $msg = $files[$i];
      my $quarantine_path = $config{'clamav_quarantine'};
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$msg",
          qw(Subject From To X-AMaViS-Alert)
        );
      my $virus = $header{'X-AMaViS-Alert'};
      ($virus) = (split (':', $virus))[1];
      $virus =~ s/[ ,\n,\r]//g;

      next if (!&clamav_quarantine_add_row ($header{'Subject'}, 
        &urlize ($virus), $msg, $virus_name, $virus, $mail_from, 
	$header{'From'}, $mail_to, $header{'To'}, "$quarantine_path/$msg", 
	$date1, $date2, \@arows));

      $count++;
    }
    # spam type
    elsif ($search_type eq 'spam' && $files[$i] =~ /^spam/)
    {
      my $msg = $files[$i];
      my $quarantine_path = $config{'clamav_quarantine'};
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$msg",
          qw(Subject From To X-Spam-Level)
        );
      my $level = ": $header{'X-Spam-Level'}";
      ($level) = (split (':', $level))[1];
      $level =~ s/[ ,\n,\r]//g;

      next if (!&clamav_quarantine_add_row ($header{'Subject'}, '', 
        $msg, '', $level, $mail_from, $header{'From'}, $mail_to, 
	$header{'To'}, "$quarantine_path/$msg", $date1, $date2, \@arows));

      $count++;
    }
  }

  # return if no result
  if ($count <= 0)
  { 
    print qq(<p><b>$text{'NO_RESULT_QUARANTINE'}</b></p>);
    return;
  }

  @arows = sort clamav_sort_by_date_quarantine_table (@arows);

  &clamav_print_quarantine_table_display ($current, $search_type, 
    \%hrow, \@arows);

  return $count;
}

# clamav_quarantine_add_row ( $ $ $ $ $ $ $ $ $ $ $ $ \@)
# IN: subject
#     url for virus search
#     basename of the file
#     virus name to search for
#     virus of the current E-Mail
#     from to search for
#     from of the current E-Mail
#     to to search for
#     to of the current E-Mail
#     location of the E-Mail file in quarantine
#     first date of the search period
#     second date of the search period
#     array reference to stock the new hash row
# OUT: 0 if bad, 1 if ok
#
# apply filters and add hash of the row items for futur display
#
sub clamav_quarantine_add_row ( $ $ $ $ $ $ $ $ $ $ $ $ \@ )
{
  my %hrow = ();
  my ($fdate, $fdate_comp) = ('', '');
  my ($subject, $url_virus, $base, $virus_name, $virus, $mail_from, $from, 
    $mail_to, $to, $file, $date1, $date2, $a) = @_;
  
  # applying filters
  return 0 if (
    ($virus_name ne '' && $virus !~ /$virus_name/i) ||
    ($mail_from ne '' && $from !~ /$mail_from/i) ||
    ($mail_to ne '' && $to !~ /$mail_to/i)
  );
    
  $fdate = strftime ("%Y-%m-%d %H:%M", localtime ((stat ($file))[9]));
  $fdate_comp = strftime ("%Y-%m-%d", localtime ((stat ($file))[9]));

  return 0 if (
    ($date1 ne '' && (&Date_Cmp ($fdate_comp, $date1) < 0 || 
      ($date2 ne '' && &Date_Cmp ($fdate_comp, $date2) > 0))) ||
    ($date2 ne '' && &Date_Cmp ($fdate_comp, $date2) > 0)
  );

  $hrow{'date'} = $fdate;
  $hrow{'subject'} = $subject;
  $hrow{'from'} = $from;
  $hrow{'to'} = $to;
  $hrow{'virus'} = $virus;
  $hrow{'url_virus'} = $url_virus;
  $hrow{'base'} = $base;
  
  push (@$a, \%hrow);

  return 1;
}

# clamav_sort_by_date_quarantine_table ( $ $ )
# IN: First item
#     Second item
# OUT: Comparison result (cmp)
#
# this function sort the result by date
# 
sub clamav_sort_by_date_quarantine_table ( $ $ )
{
  my ($a, $b) = @_;

  $a->{'date'} cmp $b->{'date'};
}

# clamav_print_quarantine_table_display ( $ $ \% \@ )
# IN: current page
#     search type ('virus', 'spam')
#     hash ref on rows items
#     array ref of rows
# OUT: -
#
# this function is shared by all functions that display quarantine
# search results
# 
sub clamav_print_quarantine_table_display ( $ $ \% \@ )
{
  my ($current, $search_type, $a, $b) = @_;
  my %hrow = %$a;
  my @arows = @$b;
  my $page_count = 0;
  my $i = 0;
  my $STRING_MAX_LEN = 30;

  if (&clamav_get_acl ('quarantine_delete') == 1 ||
    &clamav_get_acl ('quarantine_resend') == 1)
  {
    # write javascript function to select/unselect all items
    &clamav_print_javascript_check_uncheck_all (1, 'quarantine_file');

     print qq(
         <p><input id="checkuncheck" type="checkbox" onClick="check_uncheck_all (this.checked)"> <label for="checkuncheck">$text{'CHECK_UNCHECK_ALL'}</label></p>);
	 
    if (&clamav_get_acl ('quarantine_delete') == 1)
    {
      print qq(
        <p><input type="submit" name="delete" 
	          value="$text{'DELETE_SELECTED'}">);
    }
    
    if (&clamav_get_acl ('quarantine_resend') == 1)
    {
      print qq(<input type="submit" name="resend" 
                      value="$text{'RESEND_SELECTED'}">);
    }
      
    print qq(</p>);
  }

  print qq(
    <table border=1>
    <tr $tb>
    <td><b>$text{'DATE'}</b></td><td><b>$text{'SUBJECT'}</b></td>);
  if ($search_type eq 'spam') 
  {
    print "<td><b>$text{'SPAM_LEVEL'}</b></td>" 
      if (!&clamav_is_mailscanner ());
  }
  else 
  {
    print "<td><b>$text{'VIRUS'}</b></td>";
  }
  print qq(<td><b>$text{'FROM'}</b></td><td><b>$text{'TO'}</b></td>);

  if (&clamav_get_acl ('quarantine_resend') == 1 ||
    &clamav_get_acl ('quarantine_delete') == 1)
  {
    print qq(<td><b>$text{'ACTION'}</b></td>);
  }
    
  print qq(</tr>);

  # display search result
  for ($i = ceil ($current * $PAGE_ITEMS); 
    $i < $#arows + 1 && $page_count++ < $PAGE_ITEMS; $i++)
  {
    my %hrow = %{$arows[$i]};

    my $subject = &clamav_html_encode_quotes ($hrow{'subject'}); 
    my $from = &clamav_html_encode_quotes ($hrow{'from'}); 
    my $to = &clamav_html_encode_quotes ($hrow{'to'}); 
    my $virus = &clamav_html_encode_quotes ($hrow{'virus'}); 

    my $subjectc = &clamav_cut_string ($hrow{'subject'}, $STRING_MAX_LEN);
    my $fromc = &clamav_cut_string ($hrow{'from'}, $STRING_MAX_LEN);
    my $toc = &clamav_cut_string ($hrow{'to'}, $STRING_MAX_LEN);
    my $virusc = &clamav_cut_string ($hrow{'virus'}, $STRING_MAX_LEN);

    $subjectc = &clamav_html_encode ($subjectc); 
    $fromc = &clamav_html_encode ($fromc); 
    $toc = &clamav_html_encode ($toc); 
    if ($search_type eq 'spam') {$virusc = length ($virusc)}
      else {$virusc = &clamav_html_encode ($virusc)}

      print qq(
        <tr>
	<td $cb title="$text{'DATE'}: $hrow{'date'}\n$text{'FROM'}: $from\n$text{'TO'}: $to\n);
	if (($search_type eq 'spam'))
	{
          print $text{'SPAM_LEVEL'} if (!&clamav_is_mailscanner ());
	}
	else
	{
          print $text{'VIRUS'};
	}
	print qq(: $virus\n$text{'SUBJECT'}: ${subject}"><i><small><a href="quarantine_viewmail.cgi?base=$hrow{'base'}" target="_BLANK">$hrow{'date'}</small></i></td>
        <td $cb>$subjectc</a></td>);
      if ($search_type eq 'spam')
      {
        print qq(<td $cb><b><a href="http://www.dam.brown.edu/computing/SpamAssassin/faq.htm#q4" title="$text{'SPAM_LEVEL'}: $virus">$virusc</a></b></td>) if (!&clamav_is_mailscanner ());
      }
      else
      {
        print qq(<td $cb><b><a href="/clamav/vdb_search_main.cgi?search=on&virus=$hrow{'url_virus'}" title="$text{'VIRUS'}: $virus" target="_BLANK">$virusc</a></b></td>);
      }
      print qq(  <td $cb title="$text{'FROM'}: $from">$fromc</td>
        <td $cb title="$text{'TO'}: $to">$toc</td>);
	
	if (&clamav_get_acl ('quarantine_resend') == 1 || 
	&clamav_get_acl ('quarantine_delete') == 1)
	{
	  print qq(<td $cb><input type="checkbox" name="quarantine_file$i" 
	                          value="$hrow{'base'}"></td>);
	}
	else
	{
	  print qq(<td></td>);
	}
	
	print qq(</tr>);
  }

  print qq(</table>);
}

# clamav_html_encode_quotes ( $ )
# IN: the string to encode.
# OUT: a encoded string.
#
# encode quots in a string.
# 
sub clamav_html_encode_quotes ( $ )
{
  my $str = shift;

  $str =~ s/"/&quot;/g;

  return $str;
}

# clamav_html_encode ( $ )
# IN: the string to encode.
# OUT: a encoded string.
#
# encode a string.
# 
sub clamav_html_encode ( $ )
{
  my $str = shift;

  $str = HTML::Entities::encode ($str);
  $str =~ s/"/&quot;/g;

  return $str;
}

# clamav_print_javascript_check_uncheck_all ( $ $ )
# IN: index of the form in the page.
#     field name.
# OUT: -
#
# print the necessary javascript to manage check/unckeck all
# feature.
# 
sub clamav_print_javascript_check_uncheck_all ( $ $ )
{
  my ($form, $name) = @_;
  
  print qq(
    <script language="javascript">
      function check_uncheck_all (value)
      {
        for (var i = 0; i < document.forms[$form].length; i++)
	  if (document.forms[$form][i].name.indexOf ('$name') >= 0)
	    document.forms[$form][i].checked = value;
      }
    </script>
  );
}

# clamav_format_date ( $ $ $ )
# IN: The days, month and year
# OUT: The formated date (YYYY-MM-DD)
#
# Format a date
# 
sub clamav_format_date ( $ $ $ )
{
  my ($day, $month, $year) = @_;
  
  $day = int ($day);
  $month = int ($month);
  $year = int ($year);

  return '' if ($day == 0 || $year == 0);

  $month++;
  $year += 2000 if ($year < 1000);

  $date = 
    sprintf ("%u-%02u-%02u", $year, $month, $day);

  return $date;
}

# clamav_get_months_combo_options ( $ )
# IN: Default choice
# OUT: A buffer containing the SELECT options
#
# Build options for the SELECT tag with months
# 
sub clamav_get_months_combo_options ( $ )
{
  my $default = shift;
  my $i = 0;
  my $buf = '';
  
  foreach (split (' ', $text{'MONTHS_LIST'}))
  {
    $buf .= "<option value=\"$i\"" . (($i == $default) ? ' SELECTED' : '') . 
      ">$_</option>\n";
    $i++;
  }

  return $buf;
}

# clamav_get_period_chooser ( $ $ $ $ $ $ )
# IN: Defaults day1, month1, year1, day2, month2 and year2
# OUT: -
# 
# Display fields to manage preriod
# 
sub clamav_get_period_chooser ( $ $ $ $ $ $ )
{
  my ($day1, $month1, $year1, $day2, $month2, $year2) = @_;
  
  # first period
  print "<table border=0><tr>";
  print qq(<td>$text{'FROM_PERIOD'}</td><td><input type="text" name="day1" size="2" maxlength="2" value="$day1"></td><td><select name="month1">);
  print &clamav_get_months_combo_options ($month1);
  print qq(</select></td><td><input type="text" name="year1" size="4" maxlength="4" value="$year1"></td><td>);
  print &date_chooser_button ('day1', 'month1', 'year1', 1);
  print "</td></tr><tr>";
  # second period
  print qq(<td>$text{'TO_PERIOD'}</td><td><input type="text" name="day2" size="2" maxlength="2" value="$day2"></td><td><select name="month2">);
  print &clamav_get_months_combo_options ($month2);
  print qq(</select></td><td><input type="text" name="year2" size="4" maxlength="4" value="$year2"></td><td>);
  print &date_chooser_button ('day2', 'month2', 'year2', 1);
  print "</td></tr></table>";
}

# clamav_display_page_panel ( $ $ )
# IN: Current page.
#     Number of pages.
# OUT: -
#
# display the navigation panel for quarantine list.
#
sub clamav_display_page_panel ( $ $ )
{
  my ($current, $max) = @_;
  my $previous = 0;
  my $next = 0;
  my $i = 0;

  # do we need to paginate?
  return if ($max <= $PAGE_ITEMS);

  $current = int ($current);
  $limit = $max / $PAGE_ITEMS;
  $limit = ceil ($limit);
  $previous = $current - 1 if ($current > 0);
  $next = $current + 1;

  $url = 
    "search=1&" .
    "search_type=$in{'search_type'}&" .
    "virus_name=$in{'virus_name'}&" .
    "mail_from=$in{'mail_from'}&" .
    "mail_to=$in{'mail_to'}&" .
    "day1=$in{'day1'}&" .
    "day2=$in{'day2'}&" .
    "month1=$in{'month1'}&" .
    "month2=$in{'month2'}&" .
    "year1=$in{'year1'}&" .
    "year2=$in{'year2'}";

  print qq(<p><table border="0" cellspacing="2" cellpadding="2" width="40%">);
  if ($current > 0)
  {
    print qq(<tr><td valign="top" align="center" $cb width="10%">);
    print qq(<a href="quarantine_main.cgi?cp=$previous&$url">);
    print qq($text{'PREVIOUS'}</a>);
  }
  else
  {
    print qq(<tr><td valign="top" align="center" width="10%">&nbsp;);
  };
  print qq(</td><td valign="top" align="center" width="80%">);

  for ($i = 0; $i < $limit; $i++)
  {
    if ($i == $current)
    {
      print '<b>' . ($i + 1) . '</b> ';
    }
    else
    {
      print qq(<a href="quarantine_main.cgi?cp=$i&$url">);
      print $i + 1;
    }
    print qq(</a> );
  }

  if ($current < $limit - 1)
  {
    print qq(</td><td valign="top" align="center" $cb width="10%">);
    print qq(<a href="quarantine_main.cgi?cp=$next&$url">$text{'NEXT'}</a>);
  }
  else
  {
    print qq(</td><td valign="top" align="center" width="10%">&nbsp;);
  };
  print qq(
    </td></tr>
    </table></p>
  );
}

# clamav_footer ()
# IN: -
# OUT: -
#
# display versions for both this module and clamav
# 
sub clamav_footer ()
{
  my $clamav_version = &clamav_get_version ();
  my $wbmclamav_version = $module_info{'version'};
  
  print qq(
    <i>ClamAV</i> <b>$clamav_version</b> / 
    <i>wbmclamav</i> <b>$wbmclamav_version</b>
  );
}

# clamav_check_perl_deps ()
# IN: -
# OUT: -
#
# check if all dependencies are ok for perl
# -> test them before with the "eval" function and push all bad
#    dependencies in the global @perl_deps array
#
sub clamav_check_perl_deps ()
{
  return if (!scalar (@perl_deps));
                                                                                
  print qq($text{'PERL_DEPS_ERROR'});
  print qq(<ul>);
  foreach my $mod (@perl_deps) {print qq(<li><b>$mod</b></li>\n);}
  print qq(</ul><hr>);
}

# clamav_get_quarantine_infos ()
# IN: -
# OUT: a hash with quarantine informations (size, virii count, spams count).
#
# return informations about the quarantine repository.
# 
sub clamav_get_quarantine_infos ()
{
  my @tmp = ();
  my %infos = ();
  my $size = 0;
  my $virii = 0;
  my $spams = 0;

  $infos{'size'} = '0K';
  $infos{'virii'} = '0';
  $infos{'spams'} = '0';

  if (!&clamav_is_quarantine_repository_empty ())
  {
    $infos{'size'} = 
      (split (' ', `$all_path{'du'}du -sh $config{'clamav_quarantine'}`))[0];

    # amavisd-new
    if (&clamav_is_amavisd_new ())
    {
      ($infos{'virii'}, $infos{'spams'}) = &quarantine_get_infos_amavisd_new ();
    }
    # mailscanner
    elsif (&clamav_is_mailscanner ())
    {
      ($infos{'virii'}, $infos{'spams'}) = &quarantine_get_infos_mailscanner ();
    }
    # amavis-ng
    elsif (&clamav_is_amavis_ng ())
    {
      ($infos{'virii'}, $infos{'spams'}) = &quarantine_get_infos_amavis_ng ();
    }
    # milter
    elsif (&clamav_is_amavis_ng ())
    {
      ($infos{'virii'}, $infos{'spams'}) = &quarantine_get_infos_milter ();
    }

  }

  return %infos;
}

sub quarantine_get_infos_milter ()
{
  my $virii = 0;
  my $spams = 0;
  
  opendir (DIR, $config{'clamav_quarantine'});
  while (my $dir = readdir (DIR))
  {
    next if ($dir =~ /^\./);
    
    opendir (DIR1, "$config{'clamav_quarantine'}/$dir/");
    while (my $file = readdir (DIR1))
    {
      next if ($file =~ /^\./);
      $virii++;
    };
    closedir (DIR1);
  }
  closedir (DIR);
  
  return ($virii, $spams);
}

sub quarantine_get_infos_amavis_ng ()
{
  my @files = ();
  my $virii = 0;
  my $spams = 0;
  my $i = 0;
  
  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);

  for ($i = 0; $i < $#files + 1; $i += 2)
  {
    next if ($files[$i] eq '.');
    $virii++;
  }
  
  return ($virii, $spams);
}

sub quarantine_get_infos_amavisd_new ()
{
  my @files = ();
  my $virii = 0;
  my $spams = 0;
  my $i = 0;
  
  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);

  for ($i = 0; $i < $#files + 1; $i++)
  {
    if ($files[$i] =~ /^virus/)
    {
      $virii++;
    }
    elsif ($files[$i] =~ /^spam/)
    {
      $spams++;
    }
  }

  return ($virii, $spams);
}

sub quarantine_get_infos_mailscanner ()
{
  my $virii = 0;
  my $spams = 0;
  my $tmp = '';
  
  opendir (DIR, $config{'clamav_quarantine'});
  while (my $dir = readdir (DIR))
  {
    next if ($dir =~ /^\./);
    
    opendir (DIR1, "$config{'clamav_quarantine'}/$dir/");
    while (my $dir1 = readdir (DIR1))
    {
      next if ($dir1 =~ /^\./) || ($dir1 eq 'spam');
      if (open (SPAM, "$config{'clamav_quarantine'}/$dir/spam/df$dir1"))
      {
        close (SPAM);
        $spams++
      }
      else
        {$virii++}
    };
    closedir (DIR1);
  }
  closedir (DIR);

  return ($virii, $spams);
}

# clamav_display_remote_actions ($ $ $ $)
# IN: hostname
#     port
#     clamav command
#     argument for clamav command if needed
# OUT: -
#
# display a HTML table
# 
sub clamav_display_remote_actions ($ $ $ $)
{
  my ($host, $port, $action, $arg) = @_;

  print qq(
    <table border="1">
      <tr><td $cb>$text{'HOST'}</td>
      <td><input type="text" name="host" value="$host"></td></tr>
      <tr><td $cb>$text{'PORT'}</td>
      <td><input type="text" name="port" value="$port"></td></tr>
      <tr><td $cb>$text{'COMMAND'}<br><select name="action">);

  foreach my $key (sort keys %clamav_remote_actions)
  {
    my $selected = ($key eq $action);
    my $name = sprintf ("$key - %s", ($clamav_remote_actions{$key} == 1) ?
      "Argument needed ->" : "No argument");
    printf qq(<option value="$key"%s>$name</option>), ($selected) ?
      ' SELECTED' : '';
  }

  print qq(
    </select></td><td valign="bottom"><input type="text" name="arg" value="$arg"></td></tr>
    </table>);
}
1;
